@@ -2,7 +2,8 @@ from .umap_ import UMAP
 
 # Workaround: https://github.com/numba/numba/issues/3341
 import numba
-numba.config.THREADING_LAYER = 'workqueue'
+
+numba.config.THREADING_LAYER = "workqueue"
 
 import pkg_resources
 
